@@ -18,13 +18,6 @@ extern "C" {
 
 #pragma pack(push,_CRT_PACKING)
 
-#ifndef __INTERNAL_FUNC_DEFINED
-#define __INTERNAL_FUNC_DEFINED
-  typedef void (__cdecl *_PVFV)(void);
-  typedef int (__cdecl *_PIFV)(void);
-  typedef void (__cdecl *_PVFI)(int);
-#endif
-
 #if defined (SPECIAL_CRTEXE) && (defined (_DLL) || defined (__GNUC__))
   extern int _commode;
 #else
@@ -117,18 +110,6 @@ extern wchar_t *** __MINGW_IMP_SYMBOL(__winitenv);
 #if !defined(__initenv) && !defined(__arm__)
 extern char *** __MINGW_IMP_SYMBOL(__initenv);
 #define __initenv (* __MINGW_IMP_SYMBOL(__initenv))
-#endif
-
-#ifndef _acmdln
-extern char ** __MINGW_IMP_SYMBOL(_acmdln);
-#define _acmdln (* __MINGW_IMP_SYMBOL(_acmdln))
-/*  _CRTIMP extern char *_acmdln; */
-#endif
-
-#ifndef _wcmdln
-extern char ** __MINGW_IMP_SYMBOL(_wcmdln);
-#define _wcmdln (* __MINGW_IMP_SYMBOL(_wcmdln))
-/*  __CRTIMP extern wchar_t *_wcmdln; */
 #endif
 
   _CRTIMP void __cdecl _amsg_exit(int);
