@@ -23,7 +23,7 @@ int __fpclassifyl (long double _x){
       return (((hlp.ldt->lh.high & 0x7fffffff) | hlp.ldt->lh.low) == 0 ?
 	      FP_INFINITE : FP_NAN);
     return FP_NORMAL;
-#elif defined(__arm__) || defined(_ARM_)
+#elif defined(__arm__) || defined(_ARM_) || defined(__aarch64__) || defined(_ARM64_)
     return __fpclassify(_x);
 #elif defined(__i386__) || defined(_X86_)
   unsigned short sw;
