@@ -1,10 +1,10 @@
+#define MINGW_HAS_SECURE_API 1
 #include <windows.h>
 #include <malloc.h>
 #include <errno.h>
 #include <msvcrt.h>
+#include <sec_api/conio_s.h>
 
-int __cdecl _vcprintf (const char *, va_list);
-int __cdecl _vcprintf_s (const char *, va_list);
 static int __cdecl _int_vcprintf_s (const char *, va_list);
 static int __cdecl _stub (const char *, va_list);
 
