@@ -1,8 +1,6 @@
-#define _vscprintf_p SAVE__vscprintf_p
-#include <stdio.h>
-#undef _vscprintf_p
+#define MINGW_HAS_SECURE_API 1
+#include <sec_api/stdio_s.h>
 
-int __cdecl _vscprintf_p(const char *format, va_list arglist);
 int __cdecl _vscprintf_p(const char *format, va_list arglist)
 {
     return _vscprintf_p_l(format, NULL, arglist);
