@@ -1,8 +1,6 @@
-#define _vscwprintf_p SAVE__vscwprintf_p
-#include <stdio.h>
-#undef _vscwprintf_p
+#define MINGW_HAS_SECURE_API 1
+#include <sec_api/stdio_s.h>
 
-int __cdecl _vscwprintf_p(const wchar_t *format, va_list arglist);
 int __cdecl _vscwprintf_p(const wchar_t *format, va_list arglist)
 {
     return _vscwprintf_p_l(format, NULL, arglist);
