@@ -1,10 +1,9 @@
+#define MINGW_HAS_SECURE_API 1
 #include <windows.h>
 #include <malloc.h>
 #include <errno.h>
 #include <msvcrt.h>
-
-int __cdecl sprintf_s (char *, size_t, const char *, ...);
-int __cdecl vsprintf_s (char *, size_t, const char *, va_list);
+#include <sec_api/stdio_s.h>
 
 int __cdecl (*__MINGW_IMP_SYMBOL(sprintf_s))(char *, size_t, const char *,...) = sprintf_s;
 
