@@ -7,6 +7,7 @@
 #define _WINREG_
 
 #include <_mingw_unicode.h>
+#include <winapifamily.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -110,6 +111,8 @@ extern "C" {
 
 #define WIN31_CLASS NULL
 
+#if WINAPI_FAMILY_PARTITION (WINAPI_PARTITION_DESKTOP)
+
 #define RegConnectRegistry __MINGW_NAME_AW(RegConnectRegistry)
 #define RegConnectRegistryEx __MINGW_NAME_AW(RegConnectRegistryEx)
 #define RegCreateKey __MINGW_NAME_AW(RegCreateKey)
@@ -370,8 +373,8 @@ WINADVAPI LONG WINAPI RegSetKeyValueA(
 
 WINADVAPI LONG WINAPI RegSetKeyValueW(
   HKEY hKey,
-  LPCSTR lpSubKey,
-  LPCSTR lpValueName,
+  LPCWSTR lpSubKey,
+  LPCWSTR lpValueName,
   DWORD dwType,
   LPCVOID lpData,
   DWORD cbData
@@ -408,6 +411,8 @@ WINADVAPI DWORD WINAPI InitiateShutdownW(
 
 #endif /* (_WIN32_WINNT >= 0x0600) */
 
+#endif /* WINAPI_FAMILY_PARTITION (WINAPI_PARTITION_DESKTOP) */
+
 #ifdef __cplusplus
 }
 #endif
