@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011 mingw-w64 project
+   Copyright (c) 2011-2016  mingw-w64 project
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -296,7 +296,10 @@ push_pthread_mem (_pthread_v *sv)
   if (pthr_last == NULL)
     pthr_root = pthr_last = sv;
   else
+  {
     pthr_last->next = sv;
+    pthr_last = sv;
+  }
   pthread_mutex_unlock (&mtx_pthr_locked);
 }
 
@@ -1234,8 +1237,12 @@ pthread_cancel (pthread_t t)
 	  GetThreadContext(tv->h, &ctxt);
 #ifdef _M_X64
 	  ctxt.Rip = (uintptr_t) _pthread_invoke_cancel;
-#else
+#elif defined(_M_IX86)
 	  ctxt.Eip = (uintptr_t) _pthread_invoke_cancel;
+#elif defined(_M_ARM) || defined(_M_ARM64)
+	  ctxt.Pc = (uintptr_t) _pthread_invoke_cancel;
+#else
+#error Unsupported architecture
 #endif
 	  SetThreadContext (tv->h, &ctxt);
 
@@ -1374,7 +1381,7 @@ pthread_attr_getscope (const pthread_attr_t *a, int *flag)
 }
 
 int
-pthread_attr_getstackaddr (pthread_attr_t *attr, void **stack)
+pthread_attr_getstackaddr (const pthread_attr_t *attr, void **stack)
 {
   *stack = attr->stack;
   return 0;
@@ -1454,7 +1461,7 @@ pthread_setcanceltype (int type, int *oldtype)
   return 0;
 }
 
-#if defined(__i686__)
+#if defined(__i386__)
 /* Align ESP on 16-byte boundaries. */
 #  if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 2)
 __attribute__((force_align_arg_pointer))
